/**
 * DREAM Copyright (C) 2003 2004 INRIA Rhone-Alpes
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option) any
 * later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * Contact: dream@objectweb.org
 * 
 * Initial developer(s): Matthieu Leclercq Contributor(s):
 */

package helloworld;

import java.util.HashMap;
import java.util.Map;

import org.objectweb.dream.InitializationException;
import org.objectweb.dream.control.activity.Util;
import org.objectweb.dream.control.activity.task.Task;
import org.objectweb.dream.control.activity.task.TaskController;
import org.objectweb.dream.control.activity.task.thread.ThreadPoolController;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.julia.control.lifecycle.ChainedIllegalLifeCycleException;

/**
 * Client implementation, using thread pool tasks.
 */
public class PoolClientImpl extends AbstractClientImpl
{

  Task task = new ClientTask("Hello World task");

  protected void initComponent(Component componentItf)
      throws InitializationException
  {
    super.initComponent(componentItf);
    try
    {
      Map m = new HashMap();
      m.put("thread", "pool");
      m.put("threadPool.capacity", new Integer(2));
      Util.addTask(componentItf, task, m);
    }
    catch (Exception e)
    {
      throw new InitializationException(e, componentItf, "Can't add task");
    }
  }

  /**
   * @see org.objectweb.dream.AbstractComponent#startFc()
   */
  public void startFc() throws IllegalLifeCycleException
  {
    super.startFc();
    // add two threads in the thread pool.
    try
    {
      TaskController tc = (TaskController) weaveableC
          .getFcInterface("task-controller");
      ThreadPoolController threadPoolController = (ThreadPoolController) tc
          .getTaskControl(task);
      threadPoolController.addThreads(2);
    }
    catch (Exception e)
    {
      throw new ChainedIllegalLifeCycleException(e, null,
          "An error occurs while retreiving task control interface");
    }
  }
}