/**
 * DREAM
 * Copyright (C) 2003 2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.control.logger;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.julia.Util;

/**
 * Change the base name of component loggers at startup to reflect the
 * component's path. This is done only if the base name was not assigned with
 * the {@link LoggerController#setBaseName(String) }method. <br>
 * This mixin should be used only with life cycle controller that does not
 * implement
 * {@link org.objectweb.fractal.julia.control.lifecycle.LifeCycleCoordinator}
 * interface. <br>
 * <br>
 * <b>Requirements </b>
 * <ul>
 * <li>this mixin requires the
 * {@link org.objectweb.fractal.julia.UseComponentMixin }mixin.</li>
 * <li>this mixin requires the
 * {@link org.objectweb.dream.control.logger.UseLoggerControllerMixin }mixin.
 * </li>
 * <li>the component to which this controller belongs should provide the
 * {@link org.objectweb.dream.control.logger.LoggerInternalBaseName }interface.
 * </li>
 * </ul>
 */
public abstract class LoggerLifeCycleMixin
{

  // -------------------------------------------------------------------------
  // Fields and methods added and overriden by the mixin class
  // -------------------------------------------------------------------------

  /**
   * Calls the overriden method and then sets the activityLogger and
   * activityLogger factory of the user component encapsulated in this component
   * (if there is one).
   * 
   * @see org.objectweb.fractal.api.control.LifeCycleController#startFc()
   */
  public void startFc() throws IllegalLifeCycleException
  {
    StringBuffer path = new StringBuffer();
    Util.toString(_this_weaveableC, path);
    String s = path.toString().substring(1).replace('/', '.');
    if (_this_weaveableOptLogC instanceof LoggerInternalBaseName)
    {
      ((LoggerInternalBaseName) _this_weaveableOptLogC).internalSetBaseName(s);
    }
    else
    {
      _this_weaveableOptLogC.setBaseName(s);
    }
    _super_startFc();
  }

  // ---------------------------------------------------------------------------
  // Fields and methods required by the mixin class in the base class
  // ---------------------------------------------------------------------------

  /**
   * The <tt>weaveableC</tt> field required by this mixin. This field is
   * supposed to reference the {@link Component }interface of the component to
   * which this controller object belongs.
   */
  public Component        _this_weaveableC;

  /**
   * The <tt>weaveableOptLogC</tt> field required by this mixin. This field is
   * supposed to reference the {@link LoggerController }interface of the
   * component to which this controller object belongs.
   */
  public LoggerController _this_weaveableOptLogC;

  /**
   * The {@link org.objectweb.fractal.api.control.LifeCycleController#startFc()}
   * method overriden by this mixin.
   * 
   * @see org.objectweb.fractal.api.control.LifeCycleController#startFc()
   */
  public abstract void _super_startFc() throws IllegalLifeCycleException;

}