/**
 * DREAM
 * Copyright (C) 2003 2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.queue;

import java.util.Map;

import org.objectweb.dream.InterruptedPullException;
import org.objectweb.dream.Pull;
import org.objectweb.dream.PullException;
import org.objectweb.dream.message.Message;
import org.objectweb.dream.util.EmptyStringArray;

/**
 * Basic implementation of a Push/Pull buffer. By default the pull method is in
 * blocking mode, but this could be changed using the component's attribute
 * controller {@link org.objectweb.dream.queue.PushPullQueueAttributeController}.
 */
public class BasicPushPullQueueImpl extends AbstractPushQueueImpl
    implements
      Pull,
      PushPullQueueAttributeController
{

  protected boolean blockingPull = true;

  /**
   * @see org.objectweb.dream.Pull#pull(java.util.Map)
   */
  public synchronized Message pull(Map context) throws PullException
  {
    while (waitingList.isEmpty())
    {
      if (!blockingPull)
      {
        return null;
      }
      try
      {
        this.wait();
      }
      catch (InterruptedException e)
      {
        throw new InterruptedPullException(
            "Interrupted will waiting for a message to be pushed", e);
      }
    }
    return (Message) waitingList.removeFirst();
  }

  // ---------------------------------------------------------------------------
  // Implementation of PushPullQueueAttributeController interface
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.dream.queue.PushPullQueueAttributeController#setBlockingPull(boolean)
   */
  public synchronized void setBlockingPull(boolean blockingPull)
  {
    this.blockingPull = blockingPull;
    if (!blockingPull)
    {
      this.notifyAll();
    }
  }

  /**
   * @see org.objectweb.dream.queue.PushPullQueueAttributeController#getBlockingPull()
   */
  public boolean getBlockingPull()
  {
    return blockingPull;
  }

  // ---------------------------------------------------------------------------
  // Implementation of BindingController interface
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.api.control.BindingController#listFc()
   */
  public String[] listFc()
  {
    return EmptyStringArray.EMPTY_STRING_ARRAY;
  }

}